package com.springjpa;

import java.util.ArrayList;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.CriteriaSpecification;
import org.hibernate.criterion.Restrictions;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit4.SpringRunner;

import com.springjpa.model.County;
import com.springjpa.model.Town;
import com.springjpa.model.TownShip;
import com.springjpa.repo.CountyRepository;
import com.springjpa.repo.TownRepository;
import com.springjpa.repo.TownShipRepository;


@RunWith(SpringRunner.class)
@SpringBootTest
public class SpringJpaApplicationIntelligentTests {
	
	@Autowired
	private SessionFactory sessionFactory;
	
	@Autowired
	private TownRepository townRepo;

	@Autowired
	private TownShipRepository tsRepo;
	
	@Autowired
	private CountyRepository countyRepo;
	

	@BeforeClass
	public static void init()  {
	}
	
	@Ignore
	@Test
	public void contextLoads() {
	}


	@Test
	public void testTownsAll() {	
		System.out.println("\n-----testTownsAll---------");
		List<Town> towns = townRepo.findAll();
		for (Town town : towns) {
			System.out.println(town);
		}
	}

	@Ignore
	@Test
	public void testCountiesAll() {	
		System.out.println("\n-----testCountiesAll---------");
		List<County> counties = countyRepo.findAll();
		for (County county : counties) {
			System.out.println(county.getName());
		}
	}

	@Ignore
	//select varos.* FROM TELEPULES2 varos join JARAS2 jaras ON jaras.id = varos.ts_id WHERE jaras.cityname = 'Mohács'
	@Test
	public void testTownsInTownship()  {
		System.out.println("\n-----testTownsInTownship: Mohácsi járás---------");
		Session session = sessionFactory.openSession();
		Criteria c = session.createCriteria(Town.class, "varos");
		c.createAlias("varos.township", "jaras");
		c.add(Restrictions.eq("jaras.cityname", "Mohács"));
		c.setResultTransformer(CriteriaSpecification.DISTINCT_ROOT_ENTITY);
		List<Town> list = c.list();
		if (!list.isEmpty())
			for (Town town : list) {
				System.out.println(town);
			}
		else
			System.out.println("--üres--");
	}
	
	@Ignore
	//select jaras.* FROM JARAS2 jaras join TELEPULES2 telepules ON telepules.ts_id = jaras.id WHERE telepules.name = 'Hímesháza'
	/** Település alapján keressük az járást a megyében
	 * @param address benne megye és településnév
	 * @param plus az itteni szöveg ("- összes megye - ") is lehet a megye értéke 
	 * @return
	 */
	@Test
	public void testTownshipByTown() {	
		System.out.println("\n-----testTownshipByTown---------");
		Session session = sessionFactory.openSession();
		Criteria c = session.createCriteria(TownShip.class, "jaras");
		c.createAlias("jaras.towns", "telepules");
		c.add(Restrictions.eq("telepules.name", "Basal"));
		c.setResultTransformer(CriteriaSpecification.DISTINCT_ROOT_ENTITY);
		List<TownShip> list = c.list();
		if (!list.isEmpty())
			System.out.println(list.get(0));
		else
			System.out.println("--üres--");
	}	
	
	@Ignore
	@Test
	public void testTownShipsAll() {	
		System.out.println("\n-----testTownShipsAll---------");
		List<TownShip> tss = tsRepo.findAll();
		for (TownShip ts : tss) {
			System.out.println(ts);
		}
	}

	@Ignore
	@Test
	public void testTownsInTownship0()  {
		System.out.println("\n-----testTownsInTownship0: Mohácsi járás---------");
		List<TownShip> tss = tsRepo.findByName("Mohácsi Járási Hivatal");
		for (TownShip ts : tss) {
			System.out.println(ts);
		}
		long ts_id = tss.get(0).getId();
//		long ts_id = 6;
		List<Town> list = townRepo.findAllForTS(ts_id);
		if (!list.isEmpty())
			for (Town town : list) {
				System.out.println(town);
			}
		else
			System.out.println("--üres--");
	}

	
	@Ignore
	@Test
	public void testTownsInCountry() {
		System.out.println("\n-----testTownsInCountry---------");
		String NAME = "Ercsi";
		TownShip ret;
		List<County> counties = countyRepo.findAll();
		for (County county : counties) {
			ret = townRepo.findByTownAndCounty(NAME, county.getName());
			if (ret != null) {
				System.out.println(NAME + " ---> " + ret);
				break;
			}
		}
	}
	
	@Ignore
	@Test
	public void testTownShipsOfTown() {
		System.out.println("\n-----testTownShipsOfTown---------");
		TownShip ts = townRepo.findByTownAndCounty("Ercsi", "Baranya megye");
		System.out.println(ts);
		for (Town town : ts.getTowns()) {
			System.out.println("\t" + town);
		}			
	}
	
	@Ignore
	@Test
	public void testTownsOfCounty() {
		System.out.println("\n-----testTownsOfCounty---------");
		int ID = 7;
		List<Town> ret = new ArrayList<>();
		List<TownShip> tss = tsRepo.findAllForCounty(ID);
		for (TownShip ts : tss) { 
			List<Town> towns = townRepo.findAllForTS(ts.getId());
			for(Town town : towns)  {
				ret.add(town);
			}
		}
		System.out.println("Megye = " + ID);
		for (Town town : ret) {
			System.out.println("\t" + town);
		}			
		
	}


	
}
