package com.springjpa.controller;
//innen: http://javainsimpleway.com/spring-mvc-form-validation-with-custom-validator/

import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.validation.Errors;
import org.springframework.validation.ValidationUtils;
import org.springframework.validation.Validator;

import com.springjpa.io.AddressForm;
import com.springjpa.model.Town;
import com.springjpa.repo.TownRepository;



@Component
public class AddressFromValidator implements Validator {
	
	@Autowired
	TownRepository townRepo;
	

	private AddressForm addressNormed;
	
	
	
	@Override
	public boolean supports(Class<?> clazz) {
		return AddressForm.class.isAssignableFrom(clazz);
	}

	
	@Override
	public void validate(Object target, Errors errors) {
		AddressForm addr  = (AddressForm)target;
				
        String city = StringUtils.trimToNull(addr.getCity());
        addressNormed = new AddressForm();
        addressNormed.setCity(city);

        //településnév
        if (city == null)
        	errors.rejectValue("city", "ERROR_NOT_DEFINED", "A településnév nem lehet üres.");
        else
	        try {
	    		List<Town> towns = townRepo.findByName(city);
	    		if (towns == null || towns.isEmpty() || towns.size() > 1) errors.rejectValue("city", "ERROR_NOT_DEFINED", "Érvénytelen a településnév.");
			} catch (Exception e)  {
//				System.out.println(e);				//IndexOutOfBoundException
				//nem tudom, h miért kell a 2. param, de így jó :))
				boolean letters = city.chars().allMatch(Character::isLetter);
				errors.rejectValue("city", "ERROR_NOT_DEFINED", letters 
						? "Nincs az országban ilyen nevű település."
						: "Érvénytelen karakter(ek) a településnévben."	);
			}
	}
	
	public AddressForm getNormed() {
		return addressNormed;
	}

}
