package com.springjpa.controller;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Locale;
import java.util.Set;

import javax.annotation.PostConstruct;
import javax.servlet.http.HttpSession;
import javax.validation.Valid;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.servlet.config.annotation.ViewControllerRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurerAdapter;

import com.springjpa.google.GoogleMapClient;
import com.springjpa.google.LatLon;
import com.springjpa.io.Address;
import com.springjpa.io.AddressForm;
import com.springjpa.model.Town;
import com.springjpa.model.TownShip;
import com.springjpa.persistence.TableRepositoryImpl;



/** 
 * INICIÁLÁS (init): h2:mem-mel dolgozik - eldobja az eddigi táblákat; létrehoz újakat, amik persze üresek
 * FELDOLGOZÁS (searchInfo): alapellenőrzés - ha hiba, vissza a beolvasásra
 * kitölti a táblákat - ha eddig is ez a tábla volt, akkor semmi; ha nem, akkor törli a táblákat, feltölti azokat
 * keres - ha a tábla valójában üres, akkor hibajelzés (error-ba); ha nincs ilyen cím, akkor hibajelzés (error-ba); ha van, akkor result-ba feltöltés
 * továbblépés a kiírásra
 * 
 * @author eax
 *
 */
@Controller
@SessionAttributes({"actual"})
public class WebController extends WebMvcConfigurerAdapter {
	
	//naplózáshoz
	public static Logger LOG = Logger.getLogger(WebController.class);

	//azonos lat-lon esetén a módosítás
	static final double DELTA_LAT = 0.004;
	static final double DELTA_LON = 0.003;
	
	//log-ban ez a keresési eredményhez
	static final String SEP = " ==>\t";
		
	//korrekció, ha a település és a járási székhely azonos (pl. Mohácsi Járás <--> Mohács)
	static final double KORR = 0.0000461;
	
	//ez a végére kerül
	static final String ALL_COUNTY = "- összes megye -";
	
	
	//@Configuration és a @Bean kapcsolatat innen: http://stackoverflow.com/questions/26461697/spring-boot-autowired-null 
	
	@Autowired
	TableRepositoryImpl tables;
	
	//spec. validáció
	@Autowired
	AddressFromValidator inputValidator;

	
	static List<String> countyList; 

	@PostConstruct
	public void init()  {
		System.out.println("Application 'Telepules' started...");
		LOG.info("Application started.");

//		table.findAll();
		
		int status = tables.checkTables();
		if (status == 3)  {
			//megyék listája kell
			countyList = tables.getAllCountyNames(ALL_COUNTY);
			
			//közben összes tepelülés is beállítva
			if (tables.checkAllTowns()) System.out.println("Van település név-duplikáció!");
			
			//kezdő választás: "összes"; sőt: nincs is megyeválasztás :))
			tables.setCountyById(0);
			
			LOG.info("Initilization succeded.");
			System.out.println("Initilization succeded.");
		} else  {
			LOG.info("Initilization failed.");
			System.out.println("Initilization failed.");
		}
	} 

	
	
    @Override
    public void addViewControllers(ViewControllerRegistry registry) {
        registry.addViewController("/results").setViewName("results");
    }

    //kirajzolja a form-ot
    @GetMapping("/")
    public String showForm(AddressForm address, Model model, HttpSession session) {
    	model.addAttribute("dropDownDistrict", countyList);

    	AddressForm actual = (AddressForm)session.getAttribute("actual");
    	if (actual != null) model.addAttribute("addressForm", actual);
    	
    	return "index"; 	//"form";
    }


	
	
	//bejön a Submit
    @PostMapping("/")
    public String searchInfo(@Valid @ModelAttribute AddressForm addressForm, BindingResult bindingResult, Model model) {
        addressForm.setCounty(ALL_COUNTY);
//    	if (bindingResult.hasErrors()) {
//        	//alaphibák
//        	model.addAttribute("dropDownDistrict", countyList);
////        	model.addAttribute("dropDownType", typeList);       	
//            return "index";		//"form";
//        } 
    	
    	inputValidator.validate(addressForm, bindingResult);
    	if (bindingResult.hasErrors()) {
    		return "index";
    	}	

    	AddressForm address = inputValidator.getNormed();


        LOG.info(address + SEP);
        model.addAttribute("actual", address);

        address.normalize();
    	//most már a település alapján keressük az járást a megyében
    	TownShip ts = tables.getTownShipOf(address, ALL_COUNTY);        	
    	if (ts == null) {
    		model.addAttribute("error", true);
    		String errorMessage = ALL_COUNTY.equals(address.getCounty())
    				? "-- " + address.getCity() + " nevű település nem létezik az " + "országban. --"		
    				: "-- " + address.getCity() + " nevű település nem létezik " + address.getCounty() + " területén. --";
    		LOG.info(address + SEP + errorMessage);
    		
    		model.addAttribute("result", errorMessage);
    	} else {
    		model.addAttribute("error", false);
    		//megvan a járás: ts
    		Address target = new Address(ts.getCounty().getName(), ts.getCityname());
    		Address target2 = new Address(ts.getCounty().getName(), ts.getCounty().getCityname());
    		LOG.info(address.getCity() + SEP + ts.getNames() + " " + ts.getCounty().getNames());
     		//ezen túl is '- összes megye -' lesz érvényes!!
    		
    		//itt a település
    		Address home = new Address(address.getCounty(), address.getCity());
    		//a címhez hozzáadom a GPS-koordinátákat
    		GoogleMapClient client = new GoogleMapClient();
    		LatLon gps = client.getCoord(home);
    		if (gps != null)  {
    			home.setGps(gps);
    		}
    		 
    		model.addAttribute("home", home);					//==>> itt a település 
    		LOG.info(address.getCity() + SEP + home.getFull());
System.out.println("\nvaros = " + home) ;
    		model.addAttribute("result", ts.getName());			//==>> mert ez string; ui. hibajelzés is itt megy ki!

    		//ha a gps 0.0/0.0, akkor gps = null!

    		int check = checkEquality(home.getCity(), target.getCity(), target2.getCity());
    		boolean homeEqTarget = false;
    		switch (check) {
    		case 111:
    			//valójában csak egy van :))
    			target.setGps(home.getGps()); 
    			target.modGps(DELTA_LAT, DELTA_LON);
    			target2.setGps(home.getGps()); 
    			target2.modGps(DELTA_LAT, -DELTA_LON);
    			homeEqTarget = true;
    			break;
    		case 112:
    			//település és járási székhely azonos: Mohács,Mohács,Pécs
    			target.setGps(home.getGps());	
    			target.modGps(DELTA_LAT, DELTA_LON);
    			target2.setGps(client.getCoord(target2));
//    			homeEqTarget = false;
    			break;
    		case 122:
    			//a két székhely azonos: Pécsvárad,Pécs,Pécs
    			target.setGps(client.getCoord(target));
    			target2.setGps(target.getGps()); 
    			target2.modGps(DELTA_LAT, DELTA_LON);
//    			homeEqTarget = false;
    			break;
    		case 123:
    			//mindenki különböző
    			target.setGps(client.getCoord(target));
    			target2.setGps(client.getCoord(target2));
//    			homeEqTarget = false;
    		}    		
    		
    		model.addAttribute("hometarget", homeEqTarget);
    		model.addAttribute("target", target);				//==>> itt a járás
    		LOG.info(target.getCity() + SEP + target.toString());
System.out.println("jaras = " + target) ;
			model.addAttribute("target2", target2);				//==>> itt a megye
			LOG.info(target2.getCity() + SEP + target2.toString());
System.out.println("megye = " + target2) ;
System.out.println("csak1 = " + homeEqTarget) ;
			
			//továbbiak
			Set<String> others = ts.getTownNames();
			Set<String> removes = new HashSet<>();
			removes.add(target.getCity());
			removes.add(home.getCity());
			others.removeAll(removes);
			List<String> towns = new ArrayList<>(others);
			Collections.sort(towns);
			model.addAttribute("others", towns);
        }
        	
        model.addAttribute("address", address);					//==>> csak a visszalépés miatt
//        return "redirect:/results";
        return "results";
    }
       



	/** Összehasonlítja a település, járási székhely, megyei székhely neveket
     *      t    j    m          t     j      m          t        j    m        t      j      m
     * 111-->Pécs,Pécs,Pécs  112-->Mohács,Mohács,Pécs vagy 122-->Pécsvárad,Pécs,Pécs 123-->Babarc,Mohács,Pécs
     * @param home
     * @param ts
     * @param county
     * @return
     */
    private int checkEquality(String town, String ts, String county) {
    	int ret = 1;
    	if (town.equals(ts)) {
    		if (ts.equals(county)) ret = 111;
    		else ret = 112;
    	} else if (ts.equals(county)) { ret = 122;
    	} else {
    		ret = 123;
    	}
    	return ret;
    }


	//új címhez a Submit
    @PostMapping("/xxx")
    public String back2ShowForm( @ModelAttribute("actual") AddressForm address, BindingResult bindingResult, Model model) {
    	model.addAttribute("dropDownDistrict", countyList);

    	//session változó feltöltése
    	address.setCounty(ALL_COUNTY);
    	model.addAttribute("addressForm", address);
    		   	
    	return "redirect:/";
    }

	
	
	static final String WOVELS = "AÁEÉIÍOÓÖŐUÚÜŰ";
	/** "a" vagy "az" névelőt teszi ki!
	 * @param full
	 * @return
	 */
	private String az(String noun) {
		String ret = "a";
		char char0 = noun.charAt(0);
		for (int i = 0; i < WOVELS.length(); i++) {
			if (WOVELS.charAt(i) == char0) {
				ret = "az";
				break;
			}
		}
		return ret + " ";
	}


	//kirajzolja a végeredményt
    @GetMapping("/results")
    public String showResult(Model model) {
        return "results";
    }

    
//------------------- autocomp service számára ----------------- 
	/** autocomplete-hez kell; visszaadja a listát
	 * @param term
	 * @param locale
	 * @return
	 */
	@ResponseBody
	@RequestMapping(value = "/json/city", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE, headers="Accept=*/*")
	public List<SearchBean> getCityList( @RequestParam("term") String term /*, Locale locale*/ ) {
		Locale locale = new Locale("hu"); 
//System.out.println("---getTownList " +  term);	 
		List<Town> townList = null;
		List<SearchBean> suggestTownList = null;
		String townName = "";
		SearchBean searchBean = null;
		try {
			townList = new ArrayList<Town>();
			suggestTownList = new ArrayList<SearchBean>();
			townList = tables.getCityList();  // load street Objects list
//System.out.println("cityNames = " + townList.size());		 
			for ( Town town : townList ){
				townName = town.getName();
				if (townName.toLowerCase( locale ).startsWith( term.toLowerCase( locale ) ) ) {
					searchBean = new SearchBean();
					searchBean.setLabel( townName );
					searchBean.setValue( townName );
					suggestTownList.add( searchBean );
				}
			}
		} catch ( /*CustomGeneric*/Exception e ) {
//			returnResponse = new ResponseEntity<List<SearchBean>>((List<SearchBean>)null, HttpStatus.BAD_REQUEST );
		}
		return suggestTownList;
	}
	

	/** Amint a megye változik
	 * @param term az új megye, szövegesen
	 * @param locale
	 * @return
	 */
	@ResponseBody
	@RequestMapping(value = "/json/district", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE, headers="Accept=*/*")
	public String changeCounty( @RequestParam("term") String district /*, Locale locale*/ ) {
//System.out.println("---changeDistrict " + district);
		String ret = "success";
		//csak az adott megye adataiból lesz lista
		tables.setCountyByName(district, ALL_COUNTY);
		
		if ("failure".equals(ret)) tables.deleteCounty();					//deleteTables();
//System.out.println("---changeDistrict " + district + " " + ret);
		return ret;
	}

}
